/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-2001 Spencer Kimball, Peter Mattis, and others
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <cairo.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gegl.h>

#include "libpikacolor/pikacolor.h"
#include "libpikamath/pikamath.h"

#include "core-types.h"

#include "gegl/pika-babl.h"
#include "gegl/pika-gegl-loops.h"

#include "pika.h"
#include "pikachannel.h"
#include "pikacontainer.h"
#include "pikadrawable.h"
#include "pikaimage.h"
#include "pikaimage-color-profile.h"
#include "pikaimage-new.h"
#include "pikaimage-pick-color.h"
#include "pikalayer.h"
#include "pikapickable.h"


gboolean
pika_image_pick_color (PikaImage   *image,
                       GList       *drawables,
                       gint         x,
                       gint         y,
                       gboolean     show_all,
                       gboolean     sample_merged,
                       gboolean     sample_average,
                       gdouble      average_radius,
                       const Babl **sample_format,
                       gpointer     pixel,
                       PikaRGB     *color)
{
  PikaImage    *pick_image = NULL;
  PikaPickable *pickable;
  GList        *iter;
  gboolean      result;

  g_return_val_if_fail (PIKA_IS_IMAGE (image), FALSE);

  for (iter = drawables; iter; iter = iter->next)
    {
      g_return_val_if_fail (PIKA_IS_DRAWABLE (iter->data), FALSE);
      g_return_val_if_fail (pika_item_get_image (iter->data) == image,
                            FALSE);
    }

  if (sample_merged && g_list_length (drawables) == 1)
    {
      if ((PIKA_IS_LAYER (drawables->data) &&
           pika_image_get_n_layers (image) == 1) ||
          (PIKA_IS_CHANNEL (drawables->data) &&
           pika_image_get_n_channels (image) == 1))
        {
          /* Let's add a special exception when an image has only one
           * layer. This was useful in particular for indexed image as
           * it allows to pick the right index value even when "Sample
           * merged" is checked. There are more possible exceptions, but
           * we can't just take them all in considerations unless we
           * want to make code extra-complicated).
           * See #3041.
           */
          sample_merged = FALSE;
        }
    }

  if (sample_merged)
    {
      if (! show_all)
        pickable = PIKA_PICKABLE (image);
      else
        pickable = PIKA_PICKABLE (pika_image_get_projection (image));
    }
  else
    {
      gboolean free_drawables = FALSE;

      if (! drawables)
        {
          drawables = pika_image_get_selected_drawables (image);
          free_drawables = TRUE;
        }

      if (! drawables)
        return FALSE;

      if (g_list_length (drawables) == 1)
        {
          gint off_x, off_y;

          pika_item_get_offset (PIKA_ITEM (drawables->data), &off_x, &off_y);
          x -= off_x;
          y -= off_y;

          pickable = PIKA_PICKABLE (drawables->data);
        }
      else /* length > 1 */
        {
          pick_image = pika_image_new_from_drawables (image->pika, drawables, FALSE, FALSE);
          pika_container_remove (image->pika->images, PIKA_OBJECT (pick_image));

          if (! show_all)
            pickable = PIKA_PICKABLE (pick_image);
          else
            pickable = PIKA_PICKABLE (pika_image_get_projection (pick_image));
          pika_pickable_flush (pickable);
        }

      if (free_drawables)
        g_list_free (drawables);
    }

  /* Do *not* call pika_pickable_flush() here because it's too expensive
   * to call it unconditionally each time e.g. the cursor view is updated.
   * Instead, call pika_pickable_flush() in the callers if needed.
   */

  if (sample_format)
    *sample_format = pika_pickable_get_format (pickable);

  result = pika_pickable_pick_color (pickable, x, y,
                                     sample_average &&
                                     ! (show_all && sample_merged),
                                     average_radius,
                                     pixel, color);

  if (show_all && sample_merged)
    {
      PikaColorProfile *profile = pika_image_get_color_profile (image);
      const Babl *space         = NULL;
      gdouble     sample[4]     = {};
      const Babl *format;

      if (profile)
        space = pika_color_profile_get_space (profile,
                                              PIKA_COLOR_RENDERING_INTENT_RELATIVE_COLORIMETRIC,
                                              NULL);
      format = babl_format_with_space ("RaGaBaA double", space);

      if (! result)
        memset (pixel, 0, babl_format_get_bytes_per_pixel (*sample_format));

      if (sample_average)
        {
          GeglBuffer *buffer = pika_pickable_get_buffer (pickable);
          gint        radius = floor (average_radius);

          pika_gegl_average_color (buffer,
                                   GEGL_RECTANGLE (x - radius,
                                                   y - radius,
                                                   2 * radius + 1,
                                                   2 * radius + 1),
                                   FALSE, GEGL_ABYSS_NONE, format, sample);
        }

      if (! result || sample_average)
        pika_pickable_pixel_to_rgb (pickable, format, sample, color);

      result = TRUE;
    }

  if (pick_image)
    g_object_unref (pick_image);

  return result;
}
